//////////////////////////////////////////////////////////////////////
// aft_firmware_update.cpp - Device Manager Property Page for
//			AFT Firmware update.
//
// Author	:	David Rokhvarg
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "sdlacfg.h"

static void PageInit(HKEY hkey, HWND hDlg);
static int start_wan_aftup(HWND hdlg, char *firmware_file_path);
				
static char szFirmwareFile[MAX_PATH];  // buffer for file name

DLGPROC_RETURN_TYPE WINAPI AFTFirmwareUpdateDlgProc(HWND hdlg, UINT msg, WPARAM wParam, LPARAM lParam)
{
	switch (msg)
	{
	case WM_NOTIFY:
		Debug(DBG_AFTUP, "AFTUP: WM_NOTIFY");

		break;

	case WM_INITDIALOG:
		{						
			Debug(DBG_AFTUP, "AFTUP: WM_INITDIALOG");
				
			szFirmwareFile[0] = '\0';

			// Open registry for our device
			// This is done in device cfg page since it is the first page to open.
			if (okay == TRUE)
			{	
				//read the configuration from the registry and update visual controls
				PageInit(hkey, hdlg);
			}

			if (!okay)
			{					// disable everything
				EnableWindow( GetDlgItem(hdlg, IDR_NONE), FALSE);
			}
		}
		break;
		
   	case WM_DESTROY:
		{
			LPSTR infofile = (LPSTR)GetWindowLongPtr(hdlg, DWL_USER);
			if (infofile)
				GlobalFree((HGLOBAL) infofile);
		}
		break;

	case WM_SHOWWINDOW:
		//This message received each time the Dialog shown/hidden.
		//Update visual controls with latest information stored in "Globals_New_Cfg".
		Debug(DBG_AFTUP, "DBG_AFTUP:WM_SHOWWINDOW");
		set_dialog_wanpipe_name(hdlg);
		PageInit(hkey, hdlg);
		break;

    case WM_COMMAND:
		// process command notification
		switch (LOWORD(wParam))
		{
		case BUTTON_HELP:
			WinHelp(GetActiveWindow(),"WanpipePnp.hlp",HELP_CONTEXT,IDH_DRIVER_CFG);
			return TRUE;

		case BUTTON_GET_FIRMWARE_FILE:
			Debug(DBG_AFTUP, "AFTUP: BUTTON_GET_FIRMWARE_FILE\n");
			{
				OPENFILENAME ofn;       // common dialog box structure

				szFirmwareFile[0] = '\0';
				
				// Initialize OPENFILENAME
				ZeroMemory(&ofn, sizeof(OPENFILENAME));
				ofn.lStructSize = sizeof(OPENFILENAME);
				ofn.hwndOwner = hdlg;
				ofn.lpstrFile = szFirmwareFile;
				ofn.nMaxFile = sizeof(szFirmwareFile);
				ofn.lpstrFilter = "*.BIN\0";//"All\0*.*\0Text\0*.TXT\0";
				ofn.nFilterIndex = 1;
				ofn.lpstrFileTitle = NULL;
				ofn.nMaxFileTitle = 0;
				ofn.lpstrInitialDir = NULL;
				ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

				// Display the Open dialog box. 
				if (GetOpenFileName(&ofn) == TRUE){
					SetDlgItemText(hdlg, EDIT_FIRMWARE_FILE, szFirmwareFile);				
				}
			}
			return TRUE;

		case BUTTON_START_FIRMUPDATE:
			if(szFirmwareFile[0] == '\0'){
				MessageBox(hdlg, "Please provide full path to firmware file!",
					"Error", MB_OK | MB_ICONINFORMATION );
			}else{
				if(start_wan_aftup(hdlg, szFirmwareFile)){
					MessageBox(hdlg, "Firmware update failed!",
						"Error", MB_OK | MB_ICONINFORMATION );
				}
			}
			return TRUE;
		}
		break;
	}
	return FALSE;
}

///////////////////////////////////////////////////////////////////////////////

UINT CALLBACK AFTFirmwareUpdatePageCallbackProc(HWND junk, UINT msg, LPPROPSHEETPAGE p)
{
	if (msg == PSPCB_RELEASE && p->lParam)
	{
		Debug(DBG_AFTUP, "PSPCB_RELEASE : DrvPageCallbackProc()\n");
	}
	return TRUE;	// matters only for PSPCB_CREATE
}

////////////////////////////////////////////////////////////////////////////////

static void PageInit(HKEY hkey, HWND hDlg)
{
	switch(iCardType)
	{
	case SDLA_AFT:
		SetDlgItemText(hDlg, IDC_CARD_TYPE, DECODE_CARD_SUBTYPE(iCardSubType));
		break;
	}

	SetDlgItemText(hDlg, EDIT_FIRMWARE_FILE, "????");
}

//////////////////////////////////////////////////////////////////////////////
//start "wan_aftup.exe" program, which actuall does the firmware update.
//
static int start_wan_aftup(HWND hdlg, char *firmware_file_path)
{
	BOOL		creation_flag;
	STARTUPINFO lpStartupInfo;
	PROCESS_INFORMATION lpProcessInformation;
	char app_fname[MAX_PATH];
	char cmd_line[MAX_PATH];
	static char InterfaceName[TMP_BUFFER_LEN];

	//always use IF0!
	snprintf(InterfaceName, TMP_BUFFER_LEN,	"%s_IF0",
		Globals_New_Cfg.DeviceName);

	//clear lpStartupInfo structure
	memset( &lpStartupInfo, 0, sizeof(STARTUPINFO) );

	GetSystemWindowsDirectory(app_fname, MAX_PATH);						//...\WINDOWS
	_snprintf(&app_fname[strlen(app_fname)], MAX_PATH, "\\System32");	//...\WINDOWS\System32

	//create full file path
	snprintf(&app_fname[strlen(app_fname)], MAX_PATH-strlen(app_fname),
		"\\wan_aftup.exe");
	//enclose in double quotes, in case there are blank spaces in the path
	snprintf(cmd_line, MAX_PATH, "\"%s\"", app_fname);

	//add command line args
	snprintf(&cmd_line[strlen(cmd_line)], MAX_PATH-strlen(cmd_line),
		" -i %s -f \"%s\"", InterfaceName, firmware_file_path);

	Debug(DBG_AFTUP, "cmd_line: %s\n", cmd_line);

	creation_flag = CreateProcess(
			NULL,				// name of executable module
			cmd_line,			// command line string (including executable path)
			NULL,				// lpProcessAttributes 
			NULL,				// lpThreadAttributes,
			FALSE,				// handle inheritance option
			CREATE_NEW_CONSOLE,	// creation flags
			NULL,				// new environment block
			NULL,				// current directory name
			&lpStartupInfo,		// startup information
			&lpProcessInformation//process information
			);

	if(creation_flag == TRUE){
		DWORD lpExitCode = 0;

		Debug(DBG_AFTUP, "__Successfully created WAN_AFTUP.EXE process__\n");

		//Prevent user from doing ANYTHING in "Device Manager" while 'wan_afup.exe' is running,
		//by waiting for it ('wan_afup.exe') to exit.
		WaitForSingleObject(lpProcessInformation.hProcess, INFINITE); 
		Debug(DBG_AFTUP, "returned from WAN_AFTUP.EXE process\n");

		if(GetExitCodeProcess(
				lpProcessInformation.hProcess,  // handle to the process
				&lpExitCode						// termination status
				)){
			//we got app's exit status
			Debug(DBG_AFTUP, "lpExitCode: %u\n", lpExitCode);
		}

		return lpExitCode;
	}else{
		MessageBox(hdlg, "Failed to create 'WAN_AFTUP.EXE' process!",
			"Error", MB_OK | MB_ICONINFORMATION );
		return 1;
	}
}

//////////////////////////////////////////////////////////////////////////////
